from typing import TYPE_CHECKING, Any, Dict, List, Optional

from pydavinci.main import resolve_obj
from pydavinci.utils import get_resolveobjs
from pydavinci.wrappers.folder import Folder
from pydavinci.wrappers.mediapoolitem import MediaPoolItem
from pydavinci.wrappers.timeline import Timeline
from pydavinci.wrappers.timelineitem import TimelineItem

if TYPE_CHECKING:
    from pydavinci.wrappers._resolve_stubs import PyRemoteMediaPool


class MediaPool(object):
    def __init__(self) -> None:

        self._obj: PyRemoteMediaPool = (
            resolve_obj.GetProjectManager().GetCurrentProject().GetMediaPool()
        )

    @property
    def root_folder(self) -> "Folder":
        """
        Gets media pool root folder

        Returns:
            (Folder): media pool root folder
        """

        return Folder(self._obj.GetRootFolder())

    def add_subfolder(self, folder_name: str, parent_folder: "Folder") -> "Folder":
        """
        Adds subfolder ``folder_name`` into ``parent_folder``

        Args:
            folder_name (str): subfolder name
            parent_folder (Folder): parent folder object

        Returns:
            (Folder): created subfolder
        """
        return Folder(self._obj.AddSubFolder(parent_folder._obj, folder_name))

    def create_empty_timeline(self, timeline_name: str) -> "Timeline":
        """
        Creates empty timeline in current folder

        Args:
            timeline_name (str): timeline name

        Returns:
            (Timeline): created timeline
        """
        return Timeline(self._obj.CreateEmptyTimeline(timeline_name))

    def append_to_timeline(self, clips: List["MediaPoolItem"]) -> List["TimelineItem"]:
        """
        Appends a list of ``MediaPoolItem``s to current active timeline

        Args:
            clips (List[MediaPoolItem]): list of ``MediaPoolItem``s to append

        Returns:
            (List[TimelineItem]): list of inserted ``TimelineItem``s
        """
        # / TODO All types: MediaPoolItem, List[Dict], Dict
        appended = self._obj.AppendToTimeline(get_resolveobjs(clips))
        return [TimelineItem(x) for x in appended]

    def create_timeline_from_clips(self, name: str, clips: List["MediaPoolItem"]) -> "Timeline":
        """
        Creates timeline ``name`` from ``clips``

        Args:
            name (str): new timeline name
            clips (List[MediaPoolItem]): list of ``MediaPoolItem``s to use for creating the timeline

        Returns:
            (Timeline): created timeline
        """
        return Timeline(self._obj.CreateTimelineFromClips(name, get_resolveobjs(clips)))

    def import_timeline_fromfile(
        self, path: str, options: Optional[Dict[Any, Any]] = None
    ) -> "Timeline":
        """
        Imports a timeline from ``path`` with ``options``:

        ```python
            options = {
            "timelineName": str,
            # specifies the name of the timeline to be created

            "importSourceClips": bool,
            # specifies whether source clips should be imported, True by default

            "sourceClipsPath": str,
            # specifies a filesystem path to search for source clips if the media
            # is inaccessible in their original path and if "importSourceClips" is True

            "sourceClipsFolders": List[Folder]
            # to search for source clips if the media is not present in current folder
            # and if "importSourceClips" is False

            "interlaceProcessing": bool,
            # specifies whether to enable interlace processing on the imported
            # timeline being created. valid only for AAF import
            }
        ```
        Args:
            path (str): timeline file path
            options (dict, optional): Dict with import options. Defaults to `None`.

        Returns:
            (Timeline): created timeline
        """
        if not options:
            return Timeline(self._obj.ImportTimelineFromFile(path, options))
        return Timeline(self._obj.ImportTimelineFromFile(path))

    def delete_timelines(self, timelines: List["Timeline"]) -> bool:
        """
        Deletes timelines

        Args:
            timelines (List[Timeline]): list of timelines to be deleted

        Returns:
            bool: ``True`` if successful, ``False`` otherwise

        Info:
            If you want to delete only one timeline, you can wrap it in a single element list.
        """
        return self._obj.DeleteTimelines(get_resolveobjs(timelines))

    @property
    def current_folder(self) -> "Folder":
        """
        Gets current mediapool folder

        Returns:
            (Folder): current mediapool folder
        """
        return Folder(self._obj.GetCurrentFolder())

    def set_current_folder(self, folder: "Folder") -> bool:
        """
        Sets current mediapool folder

        Args:
            folder (Folder): desired ``Folder``

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.SetCurrentFolder(folder._obj)

    def delete_clips(self, clips: List["MediaPoolItem"]) -> bool:
        """
        Delete ``clips``

        Args:
            clips (List[MediaPoolItem]): list of ``MediaPoolItem``s to be deleted

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.DeleteClips(get_resolveobjs(clips))

    def delete_folders(self, folders: List["Folder"]) -> bool:
        """
        Delete ``folders``

        Args:
            folders (List[Folder]): list of ``Folder``s to be deleted

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.DeleteFolders(get_resolveobjs(folders))

    def move_clips(self, clips: List["MediaPoolItem"], folder: "Folder") -> bool:
        """
        Moves ``clips`` inside current active folder to ``folder``

        Args:
            clips (List[MediaPoolItem]): list of ``MediaPoolItem``s on current folder
            folder (Folder): destination folder

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.MoveClips(get_resolveobjs(clips), folder._obj)

    def move_folders(self, folders: List["Folder"], target_folder: "Folder") -> bool:
        """
        Move ``folders`` to ``target_folder``

        Args:
            folders (List[Folder]): List of folders to be moved. If you want to only use one folder, wrap it in a list.
            target_folder (Folder): target ``Folder``

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.MoveFolders(get_resolveobjs(folders), target_folder._obj)

    def clip_mattes(self, clip: "MediaPoolItem") -> List[str]:
        """
        Gets paths of clip mattes from ``clip``

        Args:
            clip (MediaPoolItem): clip to get mattes from

        Returns:
            (List[str]): list of file paths associated with ``clip``'s mattes
        """
        return self._obj.GetClipMatteList(clip._obj)

    def timeline_mattes(self, folder: "Folder") -> List["MediaPoolItem"]:
        """
        Gets timeline mattes inside ``folder``

        Args:
            folder (Folder): folder to get timeline mattes from

        Returns:
            (List[MediaPoolItem]): list of timeline mattes
        """
        result = self._obj.GetTimelineMatteList(folder._obj)
        return [MediaPoolItem(x) for x in result]

    def delete_mattes_by_path(self, clip: "MediaPoolItem", path: List[str]) -> bool:
        """Delete mattes based on their file paths.

        Args:
            clip (MediaPoolItem): clip to have mattes deleted
            path (List[str]): list of matte paths

        Returns:
            bool: _description_
        """
        return self._obj.DeleteClipMattes(clip._obj, path)

    def relink_clips(self, clips: List["MediaPoolItem"], parent_folder: str) -> bool:
        """
        Update the folder location of specified media pool clips with the specified folder path.

        Args:
            clips (List[MediaPoolItem]): ``MediaPoolItem``s to relink
            parent_folder (str): new parent folder

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.RelinkClips(get_resolveobjs(clips), parent_folder)

    def unlink_clips(self, clips: List["MediaPoolItem"]) -> bool:
        """
        Unlink ``clips``

        Args:
            clips (List[MediaPoolItem]): clips to be made offline

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.UnlinkClips(get_resolveobjs(clips))

    def import_media(self, paths: List[str]) -> List["MediaPoolItem"]:
        """
        Import media from ``paths``

        Args:
            paths (List[str]): list of paths containing the media

        Returns:
            (List[MediaPoolItem]): list of imported ``MediaPoolItem``s

        Image Sequences:
            Doesn't support image sequences yet.
        """
        # / TODO: Implement image sequence using ImportMedia({ClipInfo})

        imported = self._obj.ImportMedia(paths)
        return [MediaPoolItem(x) for x in imported]

    def export_metadata(
        self, file_name: str, clips: Optional[List["MediaPoolItem"]] = None
    ) -> bool:
        """
        Exports metadata of specified clips to ``file_name.csv``.
            If no clips are specified, all clips from ``MediaPool`` will be used.

        Args:
            file_name (str): _description_
            clips (List[MediaPoolItem], optional): list of clips to be processed, defaults to ``None``

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        if clips:
            return self._obj.ExportMetadata(file_name, get_resolveobjs(clips))
        else:
            return self._obj.ExportMetadata(file_name)



# type: ignore
# flake8: noqa
import imp
import os
import platform
import subprocess
import sys
import time
from collections.abc import MutableMapping
from typing import Dict, List, Optional

import psutil

from pydavinci.connect import load_fusionscript
from pydavinci.utils import default_resolve_install


class BaseResolveWrapper(object):
    def __init__(self, headless: Optional[bool] = None, path: Optional[str] = None) -> None:

        self.gui_latency = 7  # seconds

        self._obj = None
        # if davinci is not running, run it
        if not self.process_active("resolve"):
            self.launch_resolve(headless, path)
            print("vazei inner")

        self.load_fusionscript()

        import fusionscript as dvr_script

        self._obj = dvr_script.scriptapp("Resolve")
        # print(self._obj.GetProjectManager())

    @property
    def _root(self):
        import fusionscript as dvr_script

        return dvr_script.scriptapp("Resolve")

    # @staticmethod
    # def _get_project_manager(cls):
    #     return self._obj.GetProjectManager()

    # @staticmethod
    # def _get_media_storage(cls):
    #     return self._obj.GetMediaStorage()

    def load_fusionscript(self):

        WIN_ENV_VARIABLES = {
            "RESOLVE_SCRIPT_API": r"%PROGRAMDATA%\Blackmagic Design\DaVinciResolve\Support\Developer\Scripting",
            "RESOLVE_SCRIPT_LIB": r"C:\Program Files\Blackmagic Design\DaVinciResolve\fusionscript.dll",
            "PYTHONPATH": r"%PYTHONPATH%;%RESOLVE_SCRIPT_API%\Modules\\",
        }

        MAC_ENV_VARIABLES = {
            "RESOLVE_SCRIPT_API": "/Library/Application Support/Blackmagic Design/DaVinciResolve/Developer/Scripting",
            "RESOLVE_SCRIPT_LIB": "/Applications/DaVinci Resolve/DaVinciResolve.app/Contents/Libraries/Fusion/fusionscript.so",
            "PYTHONPATH": "$PYTHONPATH:$RESOLVE_SCRIPT_API/Modules/",
        }
        LINUX_ENV_VARIABLES = {
            "RESOLVE_SCRIPT_API": "/opt/resolve/Developer/Scripting",
            "RESOLVE_SCRIPT_LIB": "/opt/resolve/libs/Fusion/fusionscript.so",
            "PYTHONPATH": "$PYTHONPATH:$RESOLVE_SCRIPT_API/Modules/",
        }

        if sys.platform.startswith("win32"):
            for key in WIN_ENV_VARIABLES.keys():
                os.environ[key] = WIN_ENV_VARIABLES[key]

        elif sys.platform.startswith("darwin"):
            for key in MAC_ENV_VARIABLES.keys():
                os.environ[key] = MAC_ENV_VARIABLES[key]

        else:
            for key in LINUX_ENV_VARIABLES.keys():
                os.environ[key] = LINUX_ENV_VARIABLES[key]

        script_module = None

        try:
            import fusionscript as script_module
        except ImportError:
            # Look for installer based environment variables:
            lib_path = os.getenv("RESOLVE_SCRIPT_LIB")
            if lib_path:
                try:
                    script_module = imp.load_dynamic("fusionscript", lib_path)
                except ImportError:
                    pass
            if not script_module:
                # Look for default install locations:
                ext = ".so"
                if sys.platform.startswith("darwin"):
                    path = "/Applications/DaVinci Resolve/DaVinci Resolve.app/Contents/Libraries/Fusion/"
                elif sys.platform.startswith("win") or sys.platform.startswith("cygwin"):
                    ext = ".dll"
                    path = "C:\\Program Files\\Blackmagic Design\\DaVinci Resolve\\"
                elif sys.platform.startswith("linux"):
                    path = "/opt/resolve/libs/Fusion/"

                try:
                    script_module = imp.load_dynamic(
                        "fusionscript", path + "fusionscript" + ext
                    )  # noqa: E501, B950
                except ImportError:
                    pass

    def process_active(self, process_name: str) -> bool:

        if "wsl" in platform.uname().release.lower():
            # WSL, figure out a better place to do this error.
            raise SystemError("WSL not supported.")

        for p in psutil.process_iter():
            if (
                process_name.lower() in p.name().lower()
                or process_name.lower() + ".exe" in p.name().lower()
            ):
                print(f"Found process: {process_name}")
                return True
        return False

    def launch_resolve(self, headless: Optional[bool] = False, path: Optional[str] = None):

        if not self.process_active("resolve"):

            system: str = ""
            args: List = []

            if sys.platform.startswith("win32"):
                system = "win"
            elif sys.platform.startswith("darwin"):
                system = "mac"
            elif sys.platform.startswith("linux"):
                system = "linux"
            else:
                raise Exception("Can't find correct platform.")

            if path:
                args.append(path)
            else:
                args.append(default_resolve_install[system])

            if headless:
                args.append("-nogui")

            kwargs = {}

            try:
                if system == "win":
                    # Windows fuckery here to spawn a process without it being a child
                    # of the python interpreter
                    # https://docs.microsoft.com/pt-br/windows/win32/procthread/process-creation-flags?redirectedfrom=MSDN
                    CREATE_NEW_PROCESS_GROUP = 0x00000200
                    DETACHED_PROCESS = 0x00000008
                    kwargs.update(creationflags=DETACHED_PROCESS | CREATE_NEW_PROCESS_GROUP)
                elif system == "wsl":
                    args.insert(0, "cmd.exe")
                else:
                    kwargs.update(close_fds=True)

                subprocess.Popen(
                    args,
                    start_new_session=True,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                    # **kwargs,
                )
                print("subiu processo")
                while not self.process_active("fuscript"):
                    print("Waiting for fuscript")

                self.load_fusionscript()
                import fusionscript as dvr_script

                ready = False
                while not ready:
                    self._obj = dvr_script.scriptapp("Resolve")
                    try:
                        if dvr_script.scriptapp("Resolve") is not None:
                            ready = True
                    except AttributeError:
                        continue
                if not headless:
                    time.sleep(self.gui_latency)
                time.sleep(0.5)
                return

            except FileNotFoundError:
                print("Davinci Resolve executable not found. Please double check the path")

        print("Davinci Resolve already running... Continuing")
        return


class DavinciMarker(object):
    @property
    def markers(self) -> Dict:
        """Gets all markers

        Returns:
            Dict: dictionary with all markers

        Info:
            Markers comes formatted like so: `{frame: {data}}`
            ```python
                {157:
                {'color': 'Blue',
                'duration': 1,
                'note': '', # custom data
                'name': 'Marker 1',
                'customData': ''}}
            ```
        """

        return self._obj.GetMarkers()

    def add_marker(
        self,
        frameid: int,
        color: str,
        name: str,
        *,
        note: str = "",
        duration: int = 1,
        customdata: str = "",
    ) -> bool:
        """Adds marker

        Args:
            frameid (int): frame to insert marker
            color (str): marker color
            name (str): marker name
            note (str, optional): marker note
            duration (int, optional): marker duration.
            customdata (str, optional): marker custom data

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.AddMarker(frameid, color, name, note, duration, customdata)

    def get_custom_marker(self, customdata: str) -> Dict:
        """Returns a dict that's tagged with ``customdata``

        Args:
            customdata (str): custom data to search

        Returns:
            Dict: marker dictionary
        """
        return self._obj.GetMarkerByCustomData(customdata)

    def update_custom_marker(self, frameid: int, customdata: str) -> bool:
        """Updates custom marker at ``frameid`` with ``customdata``

        Args:
            frameid (int): frame id
            customdata (str): new custom data

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.UpdateMarkerCustomData(frameid, customdata)

    def get_marker_custom_data(self, frameid: int) -> str:
        """Gets marker custom data at ``frameid``

        Args:
            frameid (int): frame id

        Returns:
            str: custom data
        """
        return self._obj.GetMarkerCustomData(frameid)

    def delete_marker(self, *, frameid: int = 0, color: str = "", customdata: str = "") -> bool:
        """Deletes marker at ``frameid`` or with color ``color`` or with customdata ``customdata``.
        Pass one named parameter only.

        Args:
            frameid (int, optional): frame id
            color (str, optional): color
            customdata (str, optional): custom data

        Raises:
            ValueError: "You need to provide either ``frameid``, ``color`` or ``customdata``."

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        if frameid:
            return self._obj.DeleteMarkerAtFrame(frameid)
        if color:
            return self._obj.DeleteMarkersByColor(color)
        if customdata:
            return self._obj.DeleteMarkerByCustomData(customdata)
        raise ValueError("You need to provide either 'frameid', 'color' or 'customdata'")


class DavinciSettings(MutableMapping):
    def __init__(self, map):
        self.__data = map

    def __len__(self):
        return len(self.__data)

    def __iter__(self):
        return iter(self.__data)

    def __setitem__(self, k, v):
        if k not in self.__data:
            raise KeyError(k)

        self.__data[k] = v

    def __delitem__(self, k):
        raise NotImplementedError

    def __getitem__(self, k):
        return self.__data[k]

    def __contains__(self, k):
        return k in self.__data



from typing import TYPE_CHECKING, Callable, Dict, Optional, Union

import pydantic.main
from pydantic import BaseModel, validator

import pydavinci.logger as log
from pydavinci.wrappers.settings.map import SETTINGS_MAP, super_scale_transform

if TYPE_CHECKING:
    from pydantic.fields import ModelField


# monkey patch to get underscore fields
def is_valid_field(name: str) -> Optional[bool]:
    if not name.startswith("__"):
        return True
    elif name == "__root__":
        return True
    return None


pydantic.main.is_valid_field = is_valid_field  # type: ignore

_ANY = Optional[Union[str, int, bool]]


class BaseConfig(BaseModel):  # type: ignore
    @validator("*", pre=True)  # type: ignore
    def str_none_to_none(cls: "BaseModel", value: _ANY) -> _ANY:  # noqa: B902
        # Special case here before everything
        # There are some "None" strings in the data. We want to alter them to the Python None
        # so Pydantic understands it
        if value == "None" or value == "":
            return None
        return value

    @validator("*")  # type: ignore
    def set_prop_validator(
        cls: "BaseModel", v: _ANY, values: Dict[_ANY, _ANY], field: "ModelField"  # noqa: B902
    ) -> _ANY:

        # This is called on all values assignment

        return resolve_transform(cls, v, values, field)

    class Config:
        extra = "forbid"
        validate_assignment = True
        allow_population_by_field_name = True
        underscore_attrs_are_private = False


def resolve_transform(
    cls: "BaseModel", value: _ANY, values: Dict[_ANY, _ANY], field: "ModelField"
) -> _ANY:
    # Here we check if it's superscale, which needs a different transform from/to Resolve
    # we also check if _selfvalidate is False, if it is, it means we're initializing the
    # class and then we just return the value.

    # If _selfvalidate is True, we then see what transform we need to use
    # in the map.py file, which has a dict of the Fields' Aliases to which callable they need to pass through
    # We then pass to the Resolve API the Alias with the right transform function applied

    if field.alias == "superScale":
        return super_scale_transform(value)  # type: ignore

    if not values.get("_selfvalidate"):
        log.debug(f"Not sending to Resolve yet. Parsing {field.alias}")

        return value

    if values["_selfvalidate"]:
        log.debug(f"Setting '{field.alias}' to {value}")

        call: Callable[[_ANY], _ANY] = SETTINGS_MAP[field.alias]  # type: ignore
        values["_obj"].SetSetting(field.alias, call(value))  # type: ignore

        return value

    return value



from typing import TYPE_CHECKING, Dict, List

# from pydavinci.wrappers._basewrappers import BaseResolveWrapper
from pydavinci.main import resolve_obj
from pydavinci.wrappers.project import Project

if TYPE_CHECKING:
    from pydavinci.wrappers._resolve_stubs import PyRemoteProjectManager

# import fusionscript as dvr_script


class ProjectManager(object):
    # try:  ## this is for when we do auto-launch
    #     _obj = resolve_obj.GetProjectManager()  # if using this one here, everything fails
    # except AttributeError:
    #     _obj = get_resolve().GetProjectManager()  # if using this here, closing projects fail

    def __init__(self) -> None:

        self._obj: PyRemoteProjectManager = resolve_obj.GetProjectManager()

    def create_project(self, project_name: str) -> Project:
        """
        Creates a project with ``project_name``

        Args:
            project_name (str): project name

        Returns:
            Project: Project
        """
        created = self._obj.CreateProject(project_name)
        return Project(created)

    def delete_project(self, project_name: str) -> bool:
        """
        Deletes project ``project_name``

        Args:
            project_name (str): project name

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.DeleteProject(project_name)

    def load_project(self, project_name: str) -> Project:
        """
        Loads project ``project_name``

        Args:
            project_name (str): project name

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return Project(self._obj.LoadProject(project_name))

    def close_project(self, project_name: Project) -> bool:
        """
        Closes project ``project_name``

        Args:
            project_name (str): project name

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.CloseProject(project_name._obj)

    def create_folder(self, folder_name: str) -> bool:
        """
        Creates project manager folder ``folder_name``

        Args:
            folder_name (str): folder name

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.CreateFolder(folder_name)

    def delete_folder(self, folder_name: str) -> bool:
        """
        Deletes project manager folder ``folder_name``

        Args:
            folder_name (str): folder name

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.DeleteFolder(folder_name)

    @property
    def projects(self) -> List[str]:
        """
        Returns a list with project names in current project manager folder

        Returns:
            List[str]: list of project names
        """
        return self._obj.GetProjectListInCurrentFolder()

    @property
    def folders(self) -> List[str]:
        """
        Returns a list with project manager folder names

        Returns:
            List[str]: list of folder names
        """
        return self._obj.GetFolderListInCurrentFolder()

    def goto_root_folder(self) -> bool:
        """
        Goes to root project manager folder

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.GotoRootFolder()

    def goto_parent_folder(self) -> bool:
        """
        Goes to parent of current project manager folder

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.GotoParentFolder()

    @property
    def folder(self) -> str:
        """
        Returns current folder name

        Returns:
            str: folder name
        """
        return self._obj.GetCurrentFolder()

    def open_folder(self, folder_name: str) -> bool:
        """
        Open folder named ``folder_name``

        Args:
            folder_name (str): folder name

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        print("entru open folder")
        return self._obj.OpenFolder(folder_name)

    def import_project(self, path: str) -> bool:
        """
        Imports ``.drp`` project located at ``path``

        Args:
            path (str): path to ``.drp`` project

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.ImportProject(path)

    def export_project(self, project_name: str, path: str, stills_and_luts: bool = False) -> bool:
        """
        Exports project

        Args:
            project_name (str): project to be exported
            path (str): path to export to
            stills_and_luts (bool, optional): whether to export with Stills and LUTs. Defaults to False.

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.ExportProject(project_name, path, stills_and_luts)

    def restore_project(self, path: str) -> bool:
        """
        Restore project from ``path``

        Args:
            path (str): project path

        Returns:
            bool: ``True`` if successful, ``False`` otherwise
        """
        return self._obj.RestoreProject(path)

    @property
    def db(self) -> Dict[str, str]:
        """
        Gets or sets current database.
        For setting a Disk DB:
             ```python
             ProjectManager.db = {
             'DbType': 'Disk',
             'DbName': 'Local Database'
             }
             ```
        For setting a PostgresSQL db:
             ```python
             ProjectManager.db = {
             'DbType': 'PostgreSQL',
             'DbName': 'PostgresDB',
             'IpAddress': '127.0.0.1'
             }
             ```

        Args:
             db_info (dict): valid ``db_info`` dict.


        Returns:
             bool: ``True`` if successful, ``False`` otherwise

        """
        return self._obj.GetCurrentDatabase()

    @db.setter
    def db(self, db_info: Dict[str, str]) -> bool:
        """
        Sets current database according to ``db_info``

         Args:
             db_info (dict): valid ``db_info`` dict.

         Info:
             Valid dictionary:
             ```python
             ProjectManager.db = {
             'DbType': 'Disk',
             'DbName': 'Local Database'
             }
             ```
             For PostgresSQL:
             ```python
             ProjectManager.db = {
             'DbType': 'PostgreSQL',
             'DbName': 'PosgresDB',
             'IpAddress': '127.0.0.1'
             }
             ```

        Returns:
             bool: ``True`` if successful, ``False`` otherwise

        """
        return self._obj.SetCurrentDatabase(db_info)

    @property
    def db_list(self) -> List[Dict[str, str]]:
        """
        Returns list of all databases

        Returns:
            list of databases
        """
        return self._obj.GetDatabaseList()



# partially generated by datamodel-codegen:
#   filename:  settings.py
#   timestamp: 2022-04-27T02:50:45+00:00

from pathlib import Path
from typing import Any, Optional, Type, Union

from pydantic import BaseModel, DirectoryPath, Field, validator
from pydantic.fields import ModelField
from typing_extensions import Literal

import pydavinci.wrappers.settings.map as map
from pydavinci.wrappers.settings.validator import BaseConfig

## Not using the aliases because mkdocs/mkdocstrings/griffe can't infer them to build the documentation

StillsNamingPattern = Literal[
    "clipName",
    "versionName",
    "sourceTimecode",
    "timelineTimecode",
    "timelineName",
    "displayLUTName",
    "customLabel",
]

ColorScienceMode = Literal["davinciYRGB", "davinciYRGBColorManagedv2", "acescc", "acescct"]

ImageResizeMode = Literal[
    "smoother",
    "bicubic",
    "bilinear",
    "sharper",
    "box",
    "catrom",
    "cubic",
    "gaussian",
    "lanczos",
    "linear",
    "mitchell",
    "nearestNeighbor",
    "quadratic",
    "sinc",
]

OptimizedCodecs = Literal[
    "rgb",
    "dnxhd_444_12b",
    "dnxhd_hqx_12b",
    "dnxhd_hq",
    "dnxhd_sq",
    "dnxhd_lb",
    "CFHDb64a",
    "CFHDv210",
    "acph",
]


CaptureCodecs = Literal[
    "rgb",
    "dnxhd_720p145",
    "dnxhd_720p220",
    "dnxhd_720p220X",
    "dnxhd_1080i145",
    "dnxhd_1080i220",
    "dnxhd_1080i220X",
    "dnxhd_1080p36",
    "dnxhd_1080p145",
    "dnxhd_1080p220",
    "dnxhd_1080p220X",
    "dnxhd_1080p440X",
    "r210",
    "yuv",
    "v210 ",
    "acph",
]


class Audio(BaseConfig):
    capture_num_channels: int = Field(alias="audioCaptureNumChannels", ge=2, le=16, multiple_of=2)
    output_has_timecode: bool = Field(alias="audioOutputHasTimecode")
    playout_num_channels: int = Field(alias="audioPlayoutNumChannels", ge=2, le=16, multiple_of=2)
    limit_meter_align_level: int = Field(alias="limitAudioMeterAlignLevel", ge=-30, le=0)
    limit_meter_display_mode: Literal["track_source", "post_fader"] = Field(
        alias="limitAudioMeterDisplayMode"
    )
    limit_meter_high_level: int = Field(alias="limitAudioMeterHighLevel", ge=-30, le=0)
    limit_meter_lufs: str = Field(alias="limitAudioMeterLUFS")
    limit_meter_loudness_scale: Literal["ebu_18_scale", "ebu_9_scale"] = Field(
        alias="limitAudioMeterLoudnessScale"
    )
    limit_meter_low_level: int = Field(alias="limitAudioMeterLowLevel", ge=-30, le=0)

    # Putting all of them here also to avoid them going to the top of the list when calling settings.audio
    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class Color(BaseConfig):
    aces_gamut_compress_type: Optional[Literal["ACES reference gamut compress"]] = Field(
        alias="colorAcesGamutCompressType"
    )
    aces_idt: str = Field(alias="colorAcesIDT")
    aces_node_lut_processing_space: Literal["acesAp0Linear", "acesccAp1"] = Field(
        alias="colorAcesNodeLUTProcessingSpace"
    )
    aces_odt: str = Field(alias="colorAcesODT")
    gallery_stills_location: Optional[Union[DirectoryPath, str]] = Field(
        alias="colorGalleryStillsLocation"
    )
    gallery_stills_naming_custom_pattern: Optional[str] = Field(
        alias="colorGalleryStillsNamingCustomPattern"
    )
    gallery_stills_naming_enabled: bool = Field(alias="colorGalleryStillsNamingEnabled")
    gallery_stills_naming_pattern: Literal[
        "clipName",
        "versionName",
        "sourceTimecode",
        "timelineTimecode",
        "timelineName",
        "displayLUTName",
        "customLabel",
    ] = Field(alias="colorGalleryStillsNamingPattern")
    gallery_stills_naming_with_still_number: Literal["off", "suffix", "prefix"] = Field(
        alias="colorGalleryStillsNamingWithStillNumber"
    )
    keyframe_dynamics_end_profile: int = Field(alias="colorKeyframeDynamicsEndProfile", ge=0, le=11)
    keyframe_dynamics_start_profile: int = Field(
        alias="colorKeyframeDynamicsStartProfile", ge=0, le=11
    )
    color_luminance_mixer_default_zero: bool = Field(alias="colorLuminanceMixerDefaultZero")
    color_science_mode: Literal[
        "davinciYRGB", "davinciYRGBColorManagedv2", "acescc", "acescct"
    ] = Field(alias="colorScienceMode")
    color_space_input: str = Field(alias="colorSpaceInput")
    color_space_input_gamma: Optional[str] = Field(alias="colorSpaceInputGamma")
    color_space_output: str = Field(alias="colorSpaceOutput")
    color_space_output_gamma: Optional[str] = Field(alias="colorSpaceOutputGamma")
    color_space_output_gamut_mapping: Optional[str] = Field(alias="colorSpaceOutputGamutMapping")
    color_space_output_gamut_saturation_knee: float = Field(
        alias="colorSpaceOutputGamutSaturationKnee"
    )
    color_space_output_gamut_saturation_max: str = Field(alias="colorSpaceOutputGamutSaturationMax")
    color_space_output_tone_luminance_max: int = Field(alias="colorSpaceOutputToneLuminanceMax")
    color_space_output_tone_mapping: Optional[str] = Field(alias="colorSpaceOutputToneMapping")
    color_space_timeline: str = Field(alias="colorSpaceTimeline")
    color_space_timeline_gamma: Optional[str] = Field(alias="colorSpaceTimelineGamma")
    use_bgr_pixel_order_for_dpx: bool = Field(alias="colorUseBGRPixelOrderForDPX")
    use_contrast_s_curve: bool = Field(alias="colorUseContrastSCurve")
    use_legacy_log_grades: int = Field(alias="colorUseLegacyLogGrades", ge=1, le=2)
    use_local_versions_as_default: bool = Field(alias="colorUseLocalVersionsAsDefault")
    use_stereo_convergence_for_effects: bool = Field(alias="colorUseStereoConvergenceForEffects")
    version1_name: Optional[str] = Field(alias="colorVersion1Name")
    version2_name: Optional[str] = Field(alias="colorVersion2Name")
    version3_name: Optional[str] = Field(alias="colorVersion3Name")
    version4_name: Optional[str] = Field(alias="colorVersion4Name")
    version5_name: Optional[str] = Field(alias="colorVersion5Name")
    version6_name: Optional[str] = Field(alias="colorVersion6Name")
    version7_name: Optional[str] = Field(alias="colorVersion7Name")
    version8_name: Optional[str] = Field(alias="colorVersion8Name")
    version9_name: Optional[str] = Field(alias="colorVersion9Name")
    version10_name: Optional[str] = Field(alias="colorVersion10Name")
    hdr10_plus_controls_on: bool = Field(alias="hdr10PlusControlsOn")
    hdr_dolby_controls_on: bool = Field(alias="hdrDolbyControlsOn")
    hdr_dolby_master_display: Optional[str] = Field(alias="hdrDolbyMasterDisplay")
    hdr_dolby_version: Literal["4.0", "2.9"] = Field(alias="hdrDolbyVersion")
    hdr_mastering_luminance_max: int = Field(alias="hdrMasteringLuminanceMax", ge=100, le=10000)
    hdr_mastering_on: bool = Field(alias="hdrMasteringOn")
    use_ca_transform: bool = Field(alias="useCATransform")
    use_color_space_aware_grading_tools: bool = Field(alias="useColorSpaceAwareGradingTools")
    use_inverse_drt: bool = Field(alias="useInverseDRT")
    output_drt: Optional[str] = Field(alias="outputDRT")
    output_drt_sat_rolloff_limit: int = Field(alias="outputDRTSatRolloffLimit", ge=500, le=20000)
    output_drt_sat_rolloff_start: int = Field(alias="outputDRTSatRolloffStart", ge=1, le=500)
    rcm_preset_mode: Literal["HDR Rec.2020 Intermediate", "SDR Rec.709", "Custom"] = Field(
        alias="rcmPresetMode"
    )
    separate_color_space_and_gamma: bool = Field(alias="separateColorSpaceAndGamma")
    input_drt: Optional[
        Literal["None", "Simple", "Luminance Mapping", "DaVinci", "Saturation Preserving"]
    ] = Field(alias="inputDRT")
    input_drt_sat_rolloff_limit: int = Field(alias="inputDRTSatRolloffLimit", ge=500, le=20000)
    input_drt_sat_rolloff_start: int = Field(alias="inputDRTSatRolloffStart", ge=1, le=500)
    auto_color_manage: bool = Field(alias="isAutoColorManage")
    limit_broadcast_safe_levels: Literal["20_120", "10_110", "0_100"] = Field(
        alias="limitBroadcastSafeLevels"
    )
    limit_broadcast_safe: bool = Field(alias="limitBroadcastSafeOn")

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class Perf(BaseConfig):
    auto_render_cache_after_time: int = Field(alias="perfAutoRenderCacheAfterTime", ge=1, le=30)
    auto_render_cache_composite: bool = Field(alias="perfAutoRenderCacheComposite")
    auto_render_cache_enable: bool = Field(alias="perfAutoRenderCacheEnable")
    auto_render_cache_fu_effect: bool = Field(alias="perfAutoRenderCacheFuEffect")
    auto_render_cache_transition: bool = Field(alias="perfAutoRenderCacheTransition")
    cache_clips_location: Optional[Union[Path, str]] = Field(alias="perfCacheClipsLocation")
    optimised_codec: Literal[
        "rgb",
        "dnxhd_444_12b",
        "dnxhd_hqx_12b",
        "dnxhd_hq",
        "dnxhd_sq",
        "dnxhd_lb",
        "CFHDb64a",
        "CFHDv210",
        "apch",
    ] = Field(alias="perfOptimisedCodec")
    optimised_media_on: bool = Field(alias="perfOptimisedMediaOn")
    optimized_resolution_ratio: Literal[
        "original", "half", "quarter", "one_eighth", "one_sixteenth", "auto"
    ] = Field(alias="perfOptimizedResolutionRatio")
    proxy_media_on: bool = Field(alias="perfProxyMediaOn")
    proxy_resolution_ratio: Literal["original", "half", "quarter"] = Field(
        alias="perfProxyResolutionRatio"
    )
    render_cache_codec: Literal[
        "rgb",
        "dnxhd_444_12b",
        "dnxhd_hqx_12b",
        "dnxhd_hq",
        "dnxhd_sq",
        "dnxhd_lb",
        "CFHDb64a",
        "CFHDv210",
        "apch",
    ] = Field(alias="perfRenderCacheCodec")
    render_cache_mode: Literal["user", "smart", "none"] = Field(alias="perfRenderCacheMode")

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class Deck(BaseConfig):
    add32_pulldown: bool = Field(alias="videoDeckAdd32Pulldown")
    bit_depth: str = Field(alias="videoDeckBitDepth")
    format: str = Field(alias="videoDeckFormat")
    non_auto_edit_frames: bool = Field(alias="videoDeckNonAutoEditFrames")
    output_sync_source: str = Field(alias="videoDeckOutputSyncSource")
    preroll_sec: int = Field(alias="videoDeckPrerollSec", ge=2, le=8)
    sdi_configuration: str = Field(alias="videoDeckSDIConfiguration")
    use444_sdi: bool = Field(alias="videoDeckUse444SDI")
    use_auto_edit: bool = Field(alias="videoDeckUseAudoEdit")
    use_stereo_sdi: bool = Field(alias="videoDeckUseStereoSDI")

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class Capture(BaseConfig):
    codec: Optional[
        Literal[
            "rgb",
            "dnxhd_720p145",
            "dnxhd_720p220",
            "dnxhd_720p220X",
            "dnxhd_1080i145",
            "dnxhd_1080i220",
            "dnxhd_1080i220X",
            "dnxhd_1080p36",
            "dnxhd_1080p145",
            "dnxhd_1080p220",
            "dnxhd_1080p220X",
            "dnxhd_1080p440X",
            "r210",
            "yuv",
            "v210 ",
            "apch",
        ]
    ] = Field(alias="videoCaptureCodec")
    format: Optional[Literal["mov", "dpx"]] = Field(alias="videoCaptureFormat")
    ingest_handles: int = Field(alias="videoCaptureIngestHandles", ge=0, le=9999)
    location: Optional[Union[Path, str]] = Field(alias="videoCaptureLocation")
    mode: Literal["video_only", "video_audio"] = Field(alias="videoCaptureMode")

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class Playout(BaseConfig):
    audio_frames_offset: int = Field(alias="videoPlayoutAudioFramesOffset", ge=0, le=99)
    batch_head_duration: int = Field(alias="videoPlayoutBatchHeadDuration", ge=0, le=99)
    batch_tail_duration: int = Field(alias="videoPlayoutBatchTailDuration", ge=0, le=99)
    ltc_frames_offset: int = Field(alias="videoPlayoutLTCFramesOffset", ge=0, le=20)
    mode: Literal["video_only", "audio_only", "video_audio"] = Field(alias="videoPlayoutMode")
    show_ltc: bool = Field(alias="videoPlayoutShowLTC")
    show_source_timecode: bool = Field(alias="videoPlayoutShowSourceTimecode")

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class ProjectUniqueSettings(BaseConfig):
    graphics_white_level: int = Field(alias="graphicsWhiteLevel", ge=100, le=4000)
    image_deinterlace_quality: Literal["high", "normal"] = Field(alias="imageDeinterlaceQuality")
    image_enable_field_processing: bool = Field(alias="imageEnableFieldProcessing")
    image_motion_estimation_mode: Literal[
        "enhancedBetter", "enhancedFaster", "standardFaster", "standardBetter"
    ] = Field(alias="imageMotionEstimationMode")
    image_motion_estimation_range: Literal["small", "larger", "medium"] = Field(
        alias="imageMotionEstimationRange"
    )
    image_resize_mode: Literal[
        "smoother",
        "bicubic",
        "bilinear",
        "sharper",
        "box",
        "catrom",
        "cubic",
        "gaussian",
        "lanczos",
        "linear",
        "mitchell",
        "nearestNeighbor",
        "quadratic",
        "sinc",
    ] = Field(alias="imageResizeMode")
    image_resizing_gamma: str = Field(alias="imageResizingGamma")
    image_retime_interpolation: Literal["nearest", "frameBlend", "opticalFlow"] = Field(
        alias="imageRetimeInterpolation"
    )
    limit_subtitle_cpl: str = Field(alias="limitSubtitleCPL")
    limit_subtitle_caption_duration_sec: int = Field(alias="limitSubtitleCaptionDurationSec", ge=1)
    super_scale_noise_reduction: Literal["Low", "Medium", "High"] = Field(
        alias="superScaleNoiseReduction"
    )
    super_scale_sharpness: Literal["Low", "Medium", "High"] = Field(alias="superScaleSharpness")
    timeline_frame_rate_mismatch_behavior: Literal["fcp7", "fcpx", "none", "resolve"] = Field(
        alias="timelineFrameRateMismatchBehavior"
    )
    timeline_input_res_mismatch_custom_preset: Optional[str] = Field(
        alias="timelineInputResMismatchCustomPreset"
    )
    timeline_input_res_mismatch_use_custom_preset: str = Field(
        alias="timelineInputResMismatchUseCustomPreset"
    )
    timeline_output_res_mismatch_custom_preset: Optional[str] = Field(
        alias="timelineOutputResMismatchCustomPreset"
    )
    timeline_output_res_mismatch_use_custom_preset: bool = Field(
        alias="timelineOutputResMismatchUseCustomPreset"
    )
    timeline_playback_frame_rate: str = Field(alias="timelinePlaybackFrameRate")
    timeline_save_thumbs_in_project: bool = Field(alias="timelineSaveThumbsInProject")
    timeline_working_luminance: int = Field(alias="timelineWorkingLuminance")
    timeline_working_luminance_mode: str = Field(alias="timelineWorkingLuminanceMode")

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class TimelineMeta(BaseConfig):

    drop_frame_timecode: bool = Field(alias="timelineDropFrameTimecode")
    frame_rate: float = Field(alias="timelineFrameRate")
    input_res_mismatch_behavior: Literal[
        "centerCrop", "scaleToCrop", "scaleToFit", "stretch"
    ] = Field(alias="timelineInputResMismatchBehavior")
    interlace_processing: bool = Field(alias="timelineInterlaceProcessing")
    output_pixel_aspect_ratio: Literal["square", "cinemascope"] = Field(
        alias="timelineOutputPixelAspectRatio"
    )
    output_res_match_timeline_res: bool = Field(alias="timelineOutputResMatchTimelineRes")
    output_res_mismatch_behavior: Literal[
        "centerCrop", "scaleToCrop", "scaleToFit", "stretch"
    ] = Field(alias="timelineOutputResMismatchBehavior")
    output_resolution_height: int = Field(alias="timelineOutputResolutionHeight", ge=1)
    output_resolution_width: int = Field(alias="timelineOutputResolutionWidth", ge=1)
    pixel_aspect_ratio: Literal["square", "cinemascope"] = Field(alias="timelinePixelAspectRatio")
    resolution_height: int = Field(alias="timelineResolutionHeight", ge=1)
    resolution_width: int = Field(alias="timelineResolutionWidth", ge=1)

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class CommonMonitor(BaseConfig):

    monitor_bit_depth: int = Field(alias="videoMonitorBitDepth")
    monitor_format: str = Field(alias="videoMonitorFormat")
    monitor_matrix_override_for422_sdi: Literal["Rec.2020", "Rec.709", "Rec.601"] = Field(
        alias="videoMonitorMatrixOverrideFor422SDI"
    )
    monitor_sdi_configuration: Literal["quad_link", "dual_link", "single_link"] = Field(
        alias="videoMonitorSDIConfiguration"
    )
    monitor_scaling: Literal["bilinear", "basic", "Bilinear", "Basic"] = Field(
        alias="videoMonitorScaling"
    )  # dirty fix, when setting on timeline it capitalizes first letter, when on project it doesn't... davinci things
    monitor_use444_sdi: bool = Field(alias="videoMonitorUse444SDI")
    monitor_use_hdr_over_hdmi: bool = Field(alias="videoMonitorUseHDROverHDMI")
    monitor_use_level_a: bool = Field(alias="videoMonitorUseLevelA")
    monitor_use_matrix_override_for422_sdi: bool = Field(
        alias="videoMonitorUseMatrixOverrideFor422SDI"
    )
    monitor_use_stereo_sdi: bool = Field(alias="videoMonitorUseStereoSDI")

    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class CommonSettings(BaseConfig):

    super_scale: Literal["auto", "no_scaling", "2x", "3x", "4x"] = Field(alias="superScale")
    video_data_levels: Literal["Video", "Full"] = Field(alias="videoDataLevels")
    video_data_levels_retain_subblock_and_super_white_data: bool = Field(
        alias="videoDataLevelsRetainSubblockAndSuperWhiteData"
    )

    @validator("super_scale")  # type: ignore
    def superscale_validator(cls: Type["BaseModel"], value: Union[int, str], field: "ModelField") -> Optional[Union[str, int]]:  # type: ignore
        return map.super_scale_transform(value)


class TimelineUniqueSettings(BaseConfig):

    use_custom_settings: bool = Field(alias="useCustomSettings")



# type: ignore
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pydavinci",
    version="0.2.1",
    author="Pedro Labonia",
    author_email="pedromslabonia@gmail.com",
    description="A Davinci Resolve API Wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pedrolabonia/pydavinci",
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    packages=setuptools.find_packages(),
    package_data={"pydavinci": ["py.typed"]},
    python_requires="==3.6.*",
    install_requires=["loguru==0.6.0", "pydantic==1.9.0", "typing_extensions==4.1.1"],
)



from typing import TYPE_CHECKING, Any, Dict, Optional, Type, TypeVar

from pydavinci.wrappers.settings.components import (
    Audio,
    Capture,
    Color,
    CommonMonitor,
    CommonSettings,
    Deck,
    Perf,
    Playout,
    ProjectUniqueSettings,
    TimelineMeta,
    TimelineUniqueSettings,
)
from pydavinci.wrappers.settings.map import super_scale_transform

if TYPE_CHECKING:
    from pydavinci.wrappers.project import Project
    from pydavinci.wrappers.settings.validator import BaseConfig
    from pydavinci.wrappers.timeline import Timeline

    AnyModel = TypeVar("AnyModel", bound=BaseConfig)


class _ProjectMeta(ProjectUniqueSettings, CommonMonitor, CommonSettings):
    audio: Audio
    color: Color
    perf: Perf
    deck: Deck
    capture: Capture
    playout: Playout
    timeline: TimelineMeta


class _ProjectSettings(ProjectUniqueSettings, CommonMonitor, CommonSettings):
    pass


class TimelineSettings(TimelineUniqueSettings, TimelineMeta, CommonSettings, CommonMonitor):
    # putting them here so they don't appear first when user calls Timeline.settings
    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


class ProjectSettings(_ProjectMeta, _ProjectSettings):
    # putting them here so they don't appear first when user calls Proejct.settings
    _selfvalidate: Optional[bool]
    _obj: Optional[Any]


def get_appropriate_keys(pydantic_model: Type["AnyModel"], data: Dict[Any, Any]) -> Dict[Any, Any]:

    # Since our base pydantic model is set to not allow extra keys,
    # we need to filter them since from the Davinci API they are all
    # together

    _ret: Dict[str, str] = {}
    data = data
    model = pydantic_model.__fields__
    for value in model.values():
        if value.alias in data.keys():
            _ret[value.alias] = data[value.alias]

    return _ret


def get_prj_settings(obj: "Project") -> ProjectSettings:

    data = obj._obj.GetSetting()

    # First setting validation to off, we just want Pydantic to infer the types for now
    # such as transforming str: "1" into int: 1 or str: "1" to bool: True on a bool field
    data["_selfvalidate"] = False

    audio = Audio.parse_obj(get_appropriate_keys(Audio, data))
    color = Color.parse_obj(get_appropriate_keys(Color, data))
    perf = Perf.parse_obj(get_appropriate_keys(Perf, data))
    deck = Deck.parse_obj(get_appropriate_keys(Deck, data))
    capture = Capture.parse_obj(get_appropriate_keys(Capture, data))
    playout = Playout.parse_obj(get_appropriate_keys(Playout, data))
    timeline = TimelineMeta.parse_obj(get_appropriate_keys(TimelineMeta, data))

    data["superScale"] = super_scale_transform(data["superScale"])

    _projectsettings = _ProjectSettings.parse_obj(
        get_appropriate_keys(_ProjectSettings, data)
    ).dict()

    # From now on we set manually the resolve ._obj so we can call them from the validator
    # later on and avoid sending the settings to the wrong Project instance for example.

    # there's probably a better way to do this but I just
    # spent about 12 hrs non stop trying to figure out how
    # to access an instance from a validator(actually I found out about 3 or 4 ways but
    # this one works best)
    # and I'm not ready for the super duper meta programming yet

    audio._obj = obj._obj
    color._obj = obj._obj
    perf._obj = obj._obj
    deck._obj = obj._obj
    capture._obj = obj._obj
    playout._obj = obj._obj
    timeline._obj = obj._obj
    audio._selfvalidate = True
    color._selfvalidate = True
    perf._selfvalidate = True
    deck._selfvalidate = True
    capture._selfvalidate = True
    playout._selfvalidate = True
    timeline._selfvalidate = True

    const = {
        "audio": audio,
        "color": color,
        "perf": perf,
        "deck": deck,
        "capture": capture,
        "playout": playout,
        "timeline": timeline,
        **_projectsettings,
    }

    # We assemble everything and then pass it to the main ProjectSettings class
    # using pydantic's construct so we don't need to infer the types again

    _ret = ProjectSettings.construct(_fields_set=None, **const)
    _ret._obj = obj._obj
    _ret._selfvalidate = True

    return _ret


def get_tl_settings(obj: "Timeline") -> TimelineSettings:

    # Same thing as above, but way easier since we don't need complex
    # nesting like ProjectSettings.color.setting

    data: Dict[Any, Any] = obj.get_setting()  # type: ignore
    data["superScale"] = super_scale_transform(data["superScale"])

    _ret = TimelineSettings(**data)
    _ret._obj = obj._obj
    _ret._selfvalidate = True

    return _ret



from typing import TYPE_CHECKING

from pydavinci.connect import load_fusionscript

if TYPE_CHECKING:
    from pydavinci.wrappers._resolve_stubs import PyRemoteResolve


def get_resolve() -> "PyRemoteResolve":
    load_fusionscript()  # type: ignore
    import fusionscript as dvr_script  # type: ignore

    return dvr_script.scriptapp("Resolve")  # type: ignore


resolve_obj: "PyRemoteResolve" = get_resolve()
